/*==============================================================================*/
/*  Copyright (C) 2009-2014, Heartland Data inc. All Rights Reserved.           */
/*                                                                              */
/*  Title  :   Data Writing Driver                                              */
/*  FileID :   43a                                                              */
/*  Version:   1.0                                                              */
/*  Author :   HLDC                                                             */
/*==============================================================================*/

/*==============================================================================*/
/*  Desc:   Header for File IO                                                  */
/*==============================================================================*/
#include <stdio.h>
#include <sys/types.h>
#include <time.h>
#include "DTdrv.h"

/*==============================================================================*/
/*  Macro:  DT_UINT                                                             */
/*  Desc:   Please change Test Point argument type for DT10 Project setting.    */
/*==============================================================================*/
#define	DT_UINT unsigned int

/*==============================================================================*/
/*  Desc:   Test Point Parametaer                                               */
/*==============================================================================*/
#define DT_VARIABLE_BIT		0x2

/*==============================================================================*/
/*  Macro:  DT_ADD_CPU_INFO                                                     */
/*  Desc:   Please set 1 when you add CPU ID to Test Point.                     */
/*==============================================================================*/
#define DT_ADD_CPU_INFO		0

/*==============================================================================*/
/*  Macro:  DT_ADD_EVENT_INFO                                                   */
/*  Desc:   Please set 1 when you add Event Value to Test Point.                */
/*==============================================================================*/
#define	DT_ADD_EVENT_INFO	0

#if DT_ADD_CPU_INFO
#include <linux/smp.h>
#endif

static FILE *dat_file = NULL;
static int BusoutMode = 0;
unsigned int pid;
unsigned int cpu_id = 0;

/*==============================================================================*/
/*  Macro:  BusOutModeSet                                                       */
/*  Desc:   0 => open file 1 => writing 2 => close file                         */
/*==============================================================================*/
#define BusOutModeSet(mode) BusoutMode = mode

/*==============================================================================*/
/*  Func:   _TP_BusOut                                                          */
/*  Desc:   Called by Test Point                                                */
/*==============================================================================*/
void _TP_BusOut(unsigned int addr, unsigned int dat)
{
	unsigned char buff[18] = {0};
	struct timespec now;

#if DT_ADD_CPU_INFO
	cpu_id = smp_processor_id();
#endif
#if DT_ADD_EVENT_INFO
	pid = getpid();
#endif
	if (BusoutMode == 0) {
		/* Customize File Path */
		dat_file = fopen("ConvertData.DAT", "wb+");
		if (dat_file == NULL) {
			return;
		}
		BusOutModeSet(1);
	}

	/* time data */
	if (clock_gettime(CLOCK_REALTIME, &now) == -1) {
		printf("DT10 : timer error!\n");
	}

	buff[0] = (unsigned char)((dat) & 0xFF);
	buff[1] = (unsigned char)((dat >> 8) & 0xFF);
	buff[2] = (unsigned char)((addr) & 0xFF);
	buff[3] = (unsigned char)((addr >> 8) & 0xFF);
	buff[4] = (unsigned char)((addr >> 16) & 0xFF);
	/* time data */
	buff[5] = (unsigned char)((now.tv_nsec) & 0xFF);
	buff[6] = (unsigned char)((now.tv_nsec >> 8) & 0xFF);
	buff[7] = (unsigned char)((now.tv_nsec >> 16) & 0xFF);
	buff[8] = (unsigned char)((now.tv_nsec >> 24) & 0xFF);
	buff[9] = (unsigned char)((now.tv_sec) & 0xFF);
	buff[10] = (unsigned char)((now.tv_sec >> 8) & 0xFF);
	buff[11] = (unsigned char)((now.tv_sec >> 16) & 0xFF);
	buff[12] = (unsigned char)((now.tv_sec >> 24) & 0xFF);
#if DT_ADD_CPU_INFO
	/* cpu id */
	buff[13] = (unsigned char)cpu_id;
#else
	buff[13] = 0x00;
#endif
#if DT_ADD_EVENT_INFO
	/* pid */
	buff[14] = (unsigned char)((pid) & 0xFF);
	buff[15] = (unsigned char)((pid >> 8) & 0xFF);
	buff[16] = (unsigned char)((pid >> 16) & 0xFF);
	buff[17] = (unsigned char)((pid >> 24) & 0xFF);
#else
	buff[14] = 0x00;
	buff[15] = 0x00;
	buff[16] = 0x00;
	buff[17] = 0x00;
#endif

	fwrite(buff, sizeof(char), 18, dat_file);
	if (BusoutMode == 2) {
		fclose(dat_file);
	}
}

/*==============================================================================*/
/*  Func:   _TP_MemoryOutput                                                    */
/*  Desc:   Called by Variable TestPoint                                        */
/*==============================================================================*/
void _TP_MemoryOutput(unsigned int addr, unsigned int dat, void *value, unsigned int size)
{
	char buff[18];
	char *p = value;
	int i, j, packetNum, count;

	if (size > 256) size = 256;
	packetNum = (size + 7) / 8;

	_TP_BusOut((addr | DT_VARIABLE_BIT), dat);	/* Memory Dump Start */

	buff[0] = size;
	buff[1] = packetNum;
	buff[2] = 0x03;
	buff[12] = 0x00;
	buff[13] = 0x00;
	buff[14] = 0x00;
	buff[15] = 0x00;
	buff[16] = 0x00;
	buff[17] = 0x00;

	count = 0;
	for (i = 0; i < packetNum; i++) {
		buff[3] = i;
		for (j = 0; j < 8; j++, count++, p++) {
			buff[4+j] = (count < size)?*p:0x00;
		}
		fwrite(buff, sizeof(char), 18, dat_file);
	}
	if (BusoutMode == 2) {
		fclose(dat_file);
	}
}

/*==============================================================================*/
/*  JNI Function                                                                */
/*==============================================================================*/
void Java_com_heartland_block_DTdrv_OpenFile(JNIEnv *env, jobject obj)
{
	if (BusoutMode == 0) {
		/* Customize File Path */
		dat_file = fopen("ConvertData.DAT", "wb+");
		if (dat_file == NULL) {
			return;
		}
		BusOutModeSet(1);
	}
}

void Java_com_heartland_block_DTdrv_CloseFile(JNIEnv *env, jobject obj)
{
	BusOutModeSet(2);
}

void Java_com_heartland_block_DTdrv_TPBusOut(JNIEnv *env, jobject obj, jint nAddress, jint nData)
{
	_TP_BusOut(nAddress, nData);
}

void Java_com_heartland_block_DTdrv_TPMemoryOutput__IIBI(JNIEnv *env, jobject obj, jint nAddress, jint nData, jbyte theValue, jint size)
{
	_TP_MemoryOutput(nAddress, nData, (void *)&theValue, size);
}

void Java_com_heartland_block_DTdrv_TPMemoryOutput__IICI(JNIEnv *env, jobject obj, jint nAddress, jint nData, jchar theValue, jint size)
{
	_TP_MemoryOutput(nAddress, nData, (void *)&theValue, size);
}

void Java_com_heartland_block_DTdrv_TPMemoryOutput__IISI(JNIEnv *env, jobject obj, jint nAddress, jint nData, jshort theValue, jint size)
{
	_TP_MemoryOutput(nAddress, nData, (void *)&theValue, size);
}

void Java_com_heartland_block_DTdrv_TPMemoryOutput__IIII(JNIEnv *env, jobject obj, jint nAddress, jint nData, jint theValue, jint size)
{
	_TP_MemoryOutput(nAddress, nData, (void *)&theValue, size);
}

void Java_com_heartland_block_DTdrv_TPMemoryOutput__IIJI(JNIEnv *env, jobject obj, jint nAddress, jint nData, jlong theValue, jint size)
{
	_TP_MemoryOutput(nAddress, nData, (void *)&theValue, size);
}

void Java_com_heartland_block_DTdrv_TPMemoryOutput__IIZI(JNIEnv *env, jobject obj, jint nAddress, jint nData, jboolean theValue, jint size)
{
	_TP_MemoryOutput(nAddress, nData, (void *)&theValue, size);
}

void Java_com_heartland_block_DTdrv_TPMemoryOutput__II_3BII(JNIEnv *env, jobject obj, jint nAddress, jint nData, jbyteArray theValue, jint size, jint nLength)
{
	_TP_MemoryOutput(nAddress, nData, (void *)&theValue, size*nLength);
}

void Java_com_heartland_block_DTdrv_TPMemoryOutput__II_3CII(JNIEnv *env, jobject obj, jint nAddress, jint nData, jcharArray theValue, jint size, jint nLength)
{
	_TP_MemoryOutput(nAddress, nData, (void *)&theValue, size*nLength);
}

void Java_com_heartland_block_DTdrv_TPMemoryOutput__II_3SII(JNIEnv *env, jobject obj, jint nAddress, jint nData, jshortArray theValue, jint size, jint nLength)
{
	_TP_MemoryOutput(nAddress, nData, (void *)&theValue, size*nLength);
}

void Java_com_heartland_block_DTdrv_TPMemoryOutput__II_3III(JNIEnv *env, jobject obj, jint nAddress, jint nData, jintArray theValue, jint size, jint nLength)
{
	_TP_MemoryOutput(nAddress, nData, (void *)&theValue, size*nLength);
}

void Java_com_heartland_block_DTdrv_TPMemoryOutput__II_3JII(JNIEnv *env, jobject obj, jint nAddress, jint nData, jlongArray theValue, jint size, jint nLength)
{
	_TP_MemoryOutput(nAddress, nData, (void *)&theValue, size*nLength);
}

void Java_com_heartland_block_DTdrv_TPMemoryOutput__II_3ZII(JNIEnv *env, jobject obj, jint nAddress, jint nData, jbooleanArray theValue, jint size, jint nLength)
{
	_TP_MemoryOutput(nAddress, nData, (void *)&theValue, size*nLength);
}
